@@ -1,16 +1,19 @@
 #' Calculate mouse-tracking measures.
 #' 
-#' Calculate a number of mouse-tracking measures for each trajectory.
+#' Calculate a number of mouse-tracking measures for each trajectory, such as 
+#' minima, maxima, and flips for each dimension, and different measures for 
+#' curvature (e.g., \code{MAD}, \code{AD}, and \code{AUC}). More information on
+#' the different measures can be found in the Details and Values sections.
 #' 
 #' Note that some measures are only returned if distance, velocity and 
-#' acceleration are calculated using \link{mt_calculate_derivatives} before 
-#' running \code{mt_calculate_measures}. Besides, the meaning of these measures
-#' depends on the values of the arguments in \link{mt_calculate_derivatives}.
+#' acceleration are calculated using \link{mt_derivatives} before 
+#' running \code{mt_measures}. Besides, the meaning of these measures
+#' depends on the values of the arguments in \link{mt_derivatives}.
 #'   
 #' If the deviations from the idealized response trajectory have been calculated
-#' using \link{mt_calculate_deviations} before running
-#' \code{mt_calculate_measures}, the corresponding data in the trajectory array
-#' will be used. If not, \code{mt_calculate_measures} will calculate these
+#' using \link{mt_deviations} before running
+#' \code{mt_measures}, the corresponding data in the trajectory array
+#' will be used. If not, \code{mt_measures} will calculate these
 #' deviations automatically.
 #' 
 #' The calculation of most measures can be deduced directly from their 
@@ -24,7 +27,7 @@
 #' below the direct path, this is denoted by a negative value. This assumes that
 #' the complete movement in the trial was from bottom to top (i.e., the end
 #' point has a higher y-position than the start point). In case the movement was
-#' from top to bottom, \code{mt_calculate_measures} automatically flips the
+#' from top to bottom, \code{mt_measures} automatically flips the
 #' signs. Both \code{MD_above} and  \code{MD_below} are also reported
 #' separately. The \strong{average deviation} (\code{AD}) is the average of all
 #' deviations across the trial.
@@ -39,25 +42,34 @@
 #' @inheritParams mt_time_normalize
 #' @param save_as a character string specifying where the calculated measures 
 #'   should be stored.
+#' @param dimensions a character vector specifying the two dimensions in the 
+#'   trajectory array that contain the mouse positions. Usually (and by 
+#'   default), the first value in the vector corresponds to the x-positions
+#'   (\code{xpos}) and the second to the y-positions (\code{ypos}).
+#' @param timestamps a character string specifying the trajectory dimension
+#'   containing the timestamps.
 #' @param flip_threshold a numeric value specifying the distance that needs to 
-#'   be exceeded in one direction so that a change in direction counts as an x- 
-#'   or y-flip.
+#'   be exceeded in one direction so that a change in direction counts as a
+#'   flip.
 #'   
-#' @return A mousetrap data object (see \link{mt_example}) where an additional
-#'   \link{data.frame} has been added (by default called "measures") containing
-#'   the per-trial mouse-tracking measures. Each row in the data.frame
-#'   corresponds to one trajectory (the corresponding trajectory is by default
-#'   identified in the column \link{mt_id}). Each column in the data.frame
-#'   corresponds to one of the measures. If a trajectory array was provided 
-#'   directly as \code{data}, only the measures data.frame will be returned.
+#' @return A mousetrap data object (see \link{mt_example}) where an additional 
+#'   \link{data.frame} has been added (by default called "measures") containing 
+#'   the per-trial mouse-tracking measures. Each row in the data.frame 
+#'   corresponds to one trajectory (the corresponding trajectory is identified 
+#'   via the rownames and, additionally, in the column "mt_id"). Each column in 
+#'   the data.frame corresponds to one of the measures. If a trajectory array 
+#'   was provided directly as \code{data}, only the measures data.frame will be 
+#'   returned.
 #'   
-#'   The following measures are computed for each trajectory: 
-#'   \item{\link{mt_id}}{Trial ID (can be used for merging measures data.frame 
-#'   with other trial-level data)}
-#'   \item{x_max}{Maximum x-position} 
-#'   \item{x_min}{Minimum x-position}
-#'   \item{y_max}{Maximum y-position} 
-#'   \item{y_min}{Minimum y-position}
+#'   The following measures are computed for each trajectory (the labels
+#'   relating to x- and y-positions will be adapted depending on the values
+#'   specified in \code{dimensions}):
+#'   \item{mt_id}{Trial ID (can be used for merging measures data.frame with
+#'   other trial-level data)}
+#'   \item{xpos_max}{Maximum x-position} 
+#'   \item{xpos_min}{Minimum x-position}
+#'   \item{ypos_max}{Maximum y-position} 
+#'   \item{ypos_min}{Minimum y-position}
 #'   \item{MAD}{Maximum absolute deviation from the direct path connecting start
 #'   and end point of the trajectory (straight line)}
 #'   \item{MAD_time}{Time at which the maximum absolute deviation was reached
@@ -72,16 +84,16 @@
 #'   \item{AUC}{Area under curve, the geometric area between the actual
 #'   trajectory and the direct path where areas below the direct path have been
 #'   subtracted}
-#'   \item{x_flips}{Number of directional changes along x-axis} 
-#'   \item{y_flips}{Number of directional changes along y-axis} 
-#'   \item{x_reversals}{Number of crossings of the y-axis} 
-#'   \item{y_reversals}{Number of crossings of the x-axis}
+#'   \item{xpos_flips}{Number of directional changes along x-axis} 
+#'   \item{ypos_flips}{Number of directional changes along y-axis} 
+#'   \item{xpos_reversals}{Number of crossings of the y-axis} 
+#'   \item{ypos_reversals}{Number of crossings of the x-axis}
 #'   \item{RT}{Response time, the total time passed until a response was given}
 #'   \item{initiation_time}{Time passed until first mouse movement was 
 #'   initiated}
 #'   \item{idle_time}{Total time without mouse movement across the entirety of
 #'   the trial}
-#'   \item{xy_dist}{Total distance covered by the trajectory}
+#'   \item{total_dist}{Total distance covered by the trajectory}
 #'   \item{vel_max}{Maximum velocity}
 #'   \item{vel_max_time}{Time at which maximum velocity occurred first}
 #'   \item{vel_min}{Minimum velocity} 
@@ -111,48 +123,63 @@
 #' \link{mt_aggregate} and \link{mt_aggregate_per_subject} for aggregating the
 #' measures.
 #' 
-#' \link{merge} for merging data.
+#' \link[dplyr]{inner_join} for merging data using the \code{dplyr} package.
+#' 
 #' 
 #' @examples
-#' mt_example <- mt_calculate_derivatives(mt_example)
-#' mt_example <- mt_calculate_deviations(mt_example)
-#' mt_example <- mt_calculate_measures(mt_example)
+#' mt_example <- mt_derivatives(mt_example)
+#' mt_example <- mt_deviations(mt_example)
+#' mt_example <- mt_measures(mt_example)
 #' 
-#' # Merge measures with trial data (adding "_raw"
-#' # to columns already existing in the trial data)
-#' mt_example_results <- merge(
+#' # Merge measures with trial data
+#' mt_example_results <- dplyr::inner_join(
 #'   mt_example$data, mt_example$measures,
-#'   by="mt_id",suffixes=c("_raw",""))
+#'   by="mt_id")
 #'   
+#' @describeIn mt_measures Calculate mouse-tracking measures
 #' @export
-mt_calculate_measures <- function(data,
+mt_measures <- function(
+  data,
   use="trajectories", save_as="measures",
-  flip_threshold=0, show_progress=TRUE) {
-
+  dimensions=c("xpos","ypos"), timestamps="timestamps",
+  flip_threshold=0,
+  verbose=FALSE, show_progress=NULL) {
+  
+  if(is.null(show_progress)==FALSE){
+    warning("The argument show_progress is deprecated. ",
+            "Please use verbose instead.",
+            call. = FALSE)
+    verbose <- show_progress
+  }
+  
+  if(length(dimensions)!=2){
+    stop("For dimensions, exactly two trajectory dimensions have to be specified.")
+  }
+  
   # Prepare data
   trajectories <- extract_data(data=data, use=use)
-  timestamps <- mt_variable_labels["timestamps"]
-  xpos <- mt_variable_labels[["xpos"]]
-  ypos <- mt_variable_labels[["ypos"]]
-  dist <- mt_variable_labels[["dist"]]
-  vel  <- mt_variable_labels[["vel"]]
-  acc  <- mt_variable_labels[["acc"]]
-  xpos_ideal <- mt_variable_labels[["xpos_ideal"]]
-  ypos_ideal <- mt_variable_labels[["ypos_ideal"]]
-  dev_ideal <- mt_variable_labels[["dev_ideal"]]
-
+  dist <- "dist"
+  vel  <- "vel"
+  acc  <- "acc"
+  dev_ideal <- "dev_ideal"
+  
+  dim1 <- dimensions[[1]]
+  dim2 <- dimensions[[2]]
+  dim1_ideal <- paste0(dim1,"_ideal")
+  dim2_ideal <- paste0(dim2,"_ideal")
+  
   # Calculate number of logs
-  nlogs <- rowSums(!is.na(trajectories[,xpos,,drop=FALSE]))
+  nlogs <- rowSums(!is.na(trajectories[,dim1,,drop=FALSE]))
   
   # Calculate deviations if no deviations were found in the data
-  if (!all(c(dev_ideal,xpos_ideal,ypos_ideal) %in% dimnames(trajectories)[[2]])) {
-    if (show_progress) {
+  if (!all(c(dev_ideal,dim1_ideal,dim2_ideal) %in% dimnames(trajectories)[[2]])) {
+    if (verbose) {
       message("Start calculating deviations of actual from idealized response trajectory.")
     }
-    trajectories <- mt_calculate_deviations(
-      data=trajectories, show_progress = show_progress
+    trajectories <- mt_deviations(
+      data=trajectories, verbose=verbose
     )
-    if (show_progress) {
+    if (verbose) {
       message("Start calculating mouse-tracking measures.")
     }
   }
@@ -160,13 +187,13 @@ mt_calculate_measures <- function(data,
   # Setup variable matrix depending on whether timestamps are provided or not
   if (timestamps %in% dimnames(trajectories)[[2]]) {
     mt_measures <- c(
-      "x_max", "x_min", "y_max", "y_min",
+      paste0(dim1,c("_max","_min")),paste0(dim2,c("_max","_min")),
       "MAD", "MAD_time",
       "MD_above", "MD_above_time",
       "MD_below", "MD_below_time",
       "AD", "AUC",
-      "x_flips", "y_flips",
-      "x_reversals", "y_reversals",
+      paste0(dimensions,"_flips"),
+      paste0(dimensions,"_reversals"),
       "RT", "initiation_time", "idle_time"
     )
     
@@ -190,18 +217,18 @@ mt_calculate_measures <- function(data,
       "Not computing the corresponding measures."
     )
     mt_measures <- c(
-      "x_max", "x_min", "y_max", "y_min",
+      paste0(dim1,c("_max","_min")),paste0(dim2,c("_max","_min")),
       "MAD", "MD_above", "MD_below",
       "AD", "AUC",
-      "x_flips", "y_flips",
-      "x_reversals", "y_reversals"
+      paste0(dimensions,"_flips"),
+      paste0(dimensions,"_reversals")
     )
   }
   
   # Add distance, velocity and acceleration-based measures
   # if the derivatives were precomputed
   if (dist %in% dimnames(trajectories)[[2]]) {
-    mt_measures <- c(mt_measures, "xy_dist")
+    mt_measures <- c(mt_measures, "total_dist")
   }
   
   if (vel %in% dimnames(trajectories)[[2]] & timestamps %in% dimnames(trajectories)[[2]]) {
@@ -228,18 +255,20 @@ mt_calculate_measures <- function(data,
   # Iterate over trajectories and calculate measures
   for (i in 1:nrow(trajectories)) {
     
+    current_nlogs <- nlogs[i]
+    
     # Extract variables
-    current_xpos <- trajectories[i, xpos, 1:nlogs[i]]
-    current_ypos <- trajectories[i, ypos, 1:nlogs[i]]
-    current_xpos_ideal <- trajectories[i, xpos_ideal, 1:nlogs[i]]
-    current_ypos_ideal <- trajectories[i, ypos_ideal, 1:nlogs[i]]
-    current_dev_ideal <- trajectories[i, dev_ideal, 1:nlogs[i]]
+    current_dim1 <- trajectories[i, dim1, 1:current_nlogs]
+    current_dim2 <- trajectories[i, dim2, 1:current_nlogs]
+    current_dim1_ideal <- trajectories[i, dim1_ideal, 1:current_nlogs]
+    current_dim2_ideal <- trajectories[i, dim2_ideal, 1:current_nlogs]
+    current_dev_ideal <- trajectories[i, dev_ideal, 1:current_nlogs]
     
     # Calculate min and max values for x and y
-    measures[i,"x_max"] <- max(current_xpos)
-    measures[i,"x_min"] <- min(current_xpos)
-    measures[i,"y_max"] <- max(current_ypos)
-    measures[i,"y_min"] <- min(current_ypos)
+    measures[i,paste0(dim1,"_max")] <- max(current_dim1)
+    measures[i,paste0(dim1,"_min")] <- min(current_dim1)
+    measures[i,paste0(dim2,"_max")] <- max(current_dim2)
+    measures[i,paste0(dim2,"_min")] <- min(current_dim2)
     
     # Maximum absolute deviation (output including sign)
     measures[i,"MAD"] <- current_dev_ideal[which.max(abs(current_dev_ideal))]
@@ -255,43 +284,39 @@ mt_calculate_measures <- function(data,
     # Calculate average deviation from direct path
     measures[i,"AD"] <- mean(current_dev_ideal)
     
-    # Calculate area under curve
-    # Use deviation of actual trajectory as y coordinates
-    # and distance from starting point of the idealized 
-    # trajectory as x coordinate
-    dist_from_start <- sqrt(
-      (current_xpos_ideal-current_xpos_ideal[1])^2 + 
-      (current_ypos_ideal-current_ypos_ideal[1])^2
-    )
     
-    # Flip sign for points below the idealized line
-    if (current_ypos_ideal[length(current_ypos_ideal)] >= current_ypos_ideal[1]){
-      dist_from_start[current_ypos_ideal < current_ypos_ideal[1]] <- 
-        -dist_from_start[current_ypos_ideal < current_ypos_ideal[1]]
-    } else {
-      dist_from_start[current_ypos_ideal > current_ypos_ideal[1]] <- 
-        -dist_from_start[current_ypos_ideal > current_ypos_ideal[1]]
+    # Calculate area under curve
+    # the geometric area between the actual trajectory and the direct path
+    # where areas below the direct path have been subtracted
+    measures[i,"AUC"]<- pracma::polyarea(current_dim1,current_dim2)
+    
+    # Flip sign of AUC depending on direction of trajectory
+    
+    # ... flip if trajectory ended in top-right
+    if (current_dim2[current_nlogs]>current_dim2[1] & 
+        current_dim1[current_nlogs]>current_dim1[1]) {
+      measures[i,"AUC"] <- -(measures[i,"AUC"])
+    
+    # ... flip if trajectory ended in bottom-left
+    } else if (current_dim2[current_nlogs]<current_dim2[1] & 
+        current_dim1[current_nlogs]<current_dim1[1]) {
+      measures[i,"AUC"] <- -(measures[i,"AUC"])
     }
     
-    # Compute the area spanned by the x/y coordinate pairs,
-    # which is now the AUC (the coordinate system has been
-    # rotated so that the x axis corresponds to the direct
-    # path between start and end points)
-    measures[i,"AUC"]<- -pracma::polyarea(dist_from_start,current_dev_ideal)
     
     # Calculate number of x_flips and y_flips
-    measures[i,"x_flips"] <- count_changes(current_xpos, threshold=flip_threshold)
-    measures[i,"y_flips"] <- count_changes(current_ypos, threshold=flip_threshold)
+    measures[i,paste0(dim1,"_flips")] <- count_changes(current_dim1, threshold=flip_threshold)
+    measures[i,paste0(dim2,"_flips")] <- count_changes(current_dim2, threshold=flip_threshold)
     
     # Calculate x_reversals
     # number of crossings of the y-axis (ignoring points exactly on y axis)
-    yside <- current_xpos[current_xpos!=0] > 0
-    measures[i,"x_reversals"] <- sum(abs(diff(yside)))
+    yside <- current_dim1[current_dim1!=0] > 0
+    measures[i,paste0(dim1,"_reversals")] <- sum(abs(diff(yside)))
     
     # Calculate y_reversals
     # number of crossings of the x-axis (ignoring points exactly on x axis)
-    xside <- current_ypos[current_ypos != 0] > 0
-    measures[i,"y_reversals"] <- sum(abs(diff(xside)))
+    xside <- current_dim2[current_dim2 != 0] > 0
+    measures[i,paste0(dim2,"_reversals")] <- sum(abs(diff(xside)))
     
     # Check if timestamps are included and if so,
     # retrieve timestamps and calculate corresponding measures
@@ -302,8 +327,8 @@ mt_calculate_measures <- function(data,
       # If first timestamp > 0, add another with 0 to indicate phase without movement
       if (current_timestamps[1] > 0) {
         current_timestamps <- c(0, current_timestamps)
-        current_xpos <- c(current_xpos[1], current_xpos)
-        current_ypos <- c(current_ypos[1], current_ypos)
+        current_dim1 <- c(current_dim1[1], current_dim1)
+        current_dim2 <- c(current_dim2[1], current_dim2)
         current_dev_ideal <- c(current_dev_ideal[1], current_dev_ideal)
         nlogs[i] <- nlogs[i] + 1
       }
@@ -313,7 +338,7 @@ mt_calculate_measures <- function(data,
       # Calculate variables for phases with and without movement
       time_diffs <- diff(current_timestamps)
       # Indicate for each sample whether the position changed
-      pos_constant <- (diff(current_xpos) == 0) & (diff(current_ypos) == 0)
+      pos_constant <- (diff(current_dim1) == 0) & (diff(current_dim2) == 0)
 
       if (all(pos_constant == FALSE)) {
         # Continuous movement
@@ -343,7 +368,7 @@ mt_calculate_measures <- function(data,
     
     # Compute total distance covered
     if (dist %in% dimnames(trajectories)[[2]]) {
-      measures[i,"xy_dist"] <- sum(abs(trajectories[i,dist,]), na.rm=TRUE)
+      measures[i,"total_dist"] <- sum(abs(trajectories[i,dist,]), na.rm=TRUE)
     }
     
     # Velocity-based measures
@@ -354,7 +379,7 @@ mt_calculate_measures <- function(data,
       vel_max_pos <- which.max(trajectories[i,vel,])
       
       # Interpolate timestamp of maximum velocity
-      # (see mt_calculate_derivatives for logic of velocity timestamps)
+      # (see mt_derivatives for logic of velocity timestamps)
       vel_max_pos <- ifelse(vel_max_pos==1, 1, c(vel_max_pos-1, vel_max_pos))
       measures[i,"vel_max_time"] <- mean(trajectories[i,timestamps,vel_max_pos])
       
@@ -362,7 +387,7 @@ mt_calculate_measures <- function(data,
       measures[i,"vel_min"] <- min(trajectories[i,vel,], na.rm=TRUE)
       vel_min_pos <- which.min(trajectories[i,vel,])
       
-      # average timestamps (see mt_calculate_derivatives for logic of velocity timestamps)
+      # average timestamps (see mt_derivatives for logic of velocity timestamps)
       vel_min_pos <- ifelse(
         vel_min_pos == 1,
         1, c(vel_min_pos-1, vel_min_pos)
@@ -380,27 +405,40 @@ mt_calculate_measures <- function(data,
       measures[i,"acc_min_time"] <- trajectories[i,timestamps,which.min(trajectories[i,acc,])]
     }
 
-    if (show_progress & i %% 100 == 0) {
+    if (verbose & i %% 100 == 0) {
       message(paste(i, "trials completed"))
     }
   }
 
-  if (show_progress) {
+  if (verbose) {
     message(paste("all", i, "trials completed"))
   }
   
-  # Convert results to data.frame
-  results <- data.frame(row.names(trajectories))
-  colnames(results) <- mt_id
-  rownames(results) <- results[,mt_id]
-  results <- cbind(results, data.frame(measures))
-
-  
-  if (is_mousetrap_data(data)){
-    data[[save_as]] <- results
-    return(data)
-  }else{
-    return(results)
-  }
+  return(create_results(
+    data=data, results=measures, use=use, save_as=save_as,
+    ids=rownames(trajectories), overwrite=TRUE))
   
 }
+
+
+#' @describeIn mt_measures Deprecated
+#' @export
+mt_calculate_measures <- function(
+  data,
+  use="trajectories", save_as="measures",
+  dimensions=c("xpos","ypos"), timestamps="timestamps",
+  flip_threshold=0,
+  verbose=FALSE,show_progress=NULL) {
+  
+  .Deprecated("mt_measures")
+  
+  return(
+    mt_measures(
+      data=data,use=use,save_as=save_as,
+      dimensions=dimensions,timestamps=timestamps,
+      flip_threshold=flip_threshold,
+      verbose=verbose,show_progress=show_progress
+    )
+  )
+  
+}
\ No newline at end of file
